@@ -32,9 +32,11 @@ use std::thread;
 use rustc_serialize::json;
 
 use websocket::{Client, Message, WebSocketStream};
+use websocket::message::Type;
 use websocket::client::request::Url;
-use websocket::client::sender::Sender;
-use websocket::client::receiver::Receiver;
+
+use websocket::sender::Sender;
+use websocket::receiver::Receiver;
 
 use websocket::ws::sender::Sender as SenderTrait;
 use websocket::ws::receiver::Receiver as ReceiverTrait;
@@ -42,6 +44,7 @@ use websocket::ws::receiver::Receiver as ReceiverTrait;
 use std::sync::Arc;
 use std::sync::Mutex;
 
+/// The main struct responsible for the connection and events.
 #[derive(Clone)]
 pub struct ChatClient {
     nick: String,
@@ -52,6 +55,11 @@ pub struct ChatClient {
 
 impl ChatClient {
     /// Creates a new connection to hack.chat.
+    ///
+    /// ```
+    /// let mut chat = ChatClient::new("WikiBot", "programming");
+    /// // Joins ?programming with the nick "WikiBot"
+    /// ```
     pub fn new(nick: &str, channel: &str) -> ChatClient {
         let url = Url::parse("wss://hack.chat/chat-ws").unwrap();
         let request = Client::connect(url).unwrap();
@@ -65,8 +73,8 @@ impl ChatClient {
             nick: nick.to_string(),
             channel: channel.to_string()
         }).unwrap();
-        let message = Message::Text(join_packet);
-        sender.send_message(message).unwrap();
+        let message = Message::text(join_packet);
+        sender.send_message(&message).unwrap();
 
         return ChatClient {
             nick: nick.to_string(),
@@ -82,32 +90,41 @@ impl ChatClient {
     /// let mut chat = ChatClient::new("TestBot", "botDev");
     /// chat.send_message("Hello there people".to_string());
     /// ```
+    ///
+    /// ```
+    /// let mut chat = ChatClient::new("TestBot", "botDev");
+    ///
+    /// let problem_count = 99;
+    /// chat.send_message(format!("I got {} problems but Rust ain't one", problem_count));
+    /// ```
     pub fn send_message(&mut self, message: String) {
         let chat_packet = json::encode(&ChatPacketSend {
             cmd: "chat".to_string(),
             text: message
         }).unwrap();
-        let message = Message::Text(chat_packet);
-        self.sender.lock().unwrap().send_message(message).unwrap();
+        let message = Message::text(chat_packet);
+        self.sender.lock().unwrap().send_message(&message).unwrap();
     }
 
     fn send_ping(&mut self) {
         let ping_packet = json::encode(&GenericPacket {
             cmd: "ping".to_string()
         }).unwrap();
-        let message = Message::Text(ping_packet);
-        self.sender.lock().unwrap().send_message(message).unwrap();
+        let message = Message::text(ping_packet);
+        self.sender.lock().unwrap().send_message(&message).unwrap();
     }
 
+    /// Sends a stats request, which results in an Info event that has the number of connected
+    /// IPs and channels.
     pub fn send_stats_request(&mut self) {
         let stats_packet = json::encode(&GenericPacket {
             cmd: "stats".to_string()
         }).unwrap();
-        let message = Message::Text(stats_packet);
-        self.sender.lock().unwrap().send_message(message).unwrap();
+        let message = Message::text(stats_packet);
+        self.sender.lock().unwrap().send_message(&message).unwrap();
     }
 
-    /// Starts the ping thread, whichs sends regular pings to keep the connection open.
+    /// Starts the ping thread, which sends regular pings to keep the connection open.
     pub fn start_ping_thread(&mut self) {
         let mut chat_clone = self.clone();
         thread::spawn(move|| {
@@ -119,6 +136,24 @@ impl ChatClient {
     }
 
     /// Returns an iterator of hack.chat events such as messages.
+    ///
+    /// #Examples
+    /// ```
+    /// let mut chat = ChatClient::new("GreetingBot", "botDev");
+    /// chat.start_ping_thread(); //Start the ping thread so we keep connected
+    ///
+    /// for event in chat.iter() {
+    ///     match event {
+    ///         ChatEvent::JoinRoom(nick) => {
+    ///             chat.send_message(format!("Welcome to the chat {}!", nick));
+    ///         },
+    ///         ChatEvent::LeaveRoom(nick) => {
+    ///             chat.send_message(format!("Goodbye {}, see you later!", nick));
+    ///         },
+    ///         _ => {}
+    ///     }
+    /// }
+    /// ```
     pub fn iter(&mut self) -> ChatClient {
         return self.clone();
     }
@@ -128,16 +163,18 @@ impl Iterator for ChatClient {
     type Item = ChatEvent;
     fn next(&mut self) -> Option<ChatEvent> {
         loop {
-            let message = match self.receiver.lock().unwrap().recv_message() {
+            let message: Message = match self.receiver.lock().unwrap().recv_message() {
                 Ok(message) => message,
                 Err(e) => {
                     println!("{}", e);
                     continue;
                 }
             };
-            match message {
-                Message::Text(data) => {
-                    let cmdpacket: GenericPacket = match json::decode(&data) {
+            
+            match message.opcode {
+                Type::Text => {
+                    let data = std::str::from_utf8(&*message.payload).unwrap();
+                    let cmdpacket: GenericPacket = match json::decode(data) {
                         Ok(cmdpacket) => cmdpacket,
                         Err(e) => {
                             println!("{}", e);
@@ -176,8 +213,8 @@ impl Iterator for ChatClient {
                         continue;
                     }
                 },
-                Message::Ping(data) => {
-                    self.sender.lock().unwrap().send_message(Message::Pong(data)).unwrap();
+                Type::Ping => {
+                    self.sender.lock().unwrap().send_message(&Message::pong(message.payload)).unwrap();
                 },
                 _ => {
                     return None;
@@ -191,6 +228,7 @@ impl Iterator for ChatClient {
 /// Various Hack.chat events
 pub enum ChatEvent {
     /// Raised when there is a new message from the channel
+    ///
     /// The format is ChatEvent::Message(nick, text, trip_code)
     Message (String, String, String),
     /// Rasied when someone joins the channel
